@@ -44,8 +44,10 @@ namespace QuantLib {
       nominalTermStructure_(std::move(nominalTermStructure)) {
 
         std::pair<Date, Date> limStart = inflationPeriod(maturity_ - swapObsLag_, zii_->frequency());
+        std::pair<Date, Date> interpolationPeriod = inflationPeriod(maturity, zii_->frequency());
 
-        if (detail::CPI::effectiveInterpolationType(zii_, observationInterpolation_) == CPI::Linear) {
+        if ((detail::CPI::effectiveInterpolationType(observationInterpolation_) == CPI::Linear) &&
+            (maturity > interpolationPeriod.first)) {
             // if interpolated, we need to cover the end of the interpolation period
             earliestDate_ = limStart.first;
             latestDate_ = limStart.second + 1;
@@ -58,7 +60,7 @@ namespace QuantLib {
         // check that the observation lag of the swap
         // is compatible with the availability lag of the index AND
         // it's interpolation (assuming the start day is spot)
-        if (detail::CPI::effectiveInterpolationType(zii_, observationInterpolation_) == CPI::Linear) {
+        if (detail::CPI::effectiveInterpolationType(observationInterpolation_) == CPI::Linear) {
             Period pShift(zii_->frequency());
             QL_REQUIRE(swapObsLag_ - pShift >= zii_->availabilityLag(),
                        "inconsistency between swap observation lag "
@@ -72,7 +74,7 @@ namespace QuantLib {
 
 
     Real ZeroCouponInflationSwapHelper::impliedQuote() const {
-        zciis_->recalculate();
+        zciis_->deepUpdate();
         return zciis_->fairRate();
     }
 
@@ -93,13 +95,8 @@ namespace QuantLib {
 
         ext::shared_ptr<ZeroInflationIndex> new_zii = zii_->clone(zits);
 
-        QL_DEPRECATED_DISABLE_WARNING
-        Handle<YieldTermStructure> nominalTS =
-            !nominalTermStructure_.empty() ? nominalTermStructure_ : z->nominalTermStructure();
-        QL_DEPRECATED_ENABLE_WARNING
-
         Real nominal = 1000000.0; // has to be something but doesn't matter what
-        Date start = nominalTS->referenceDate();
+        Date start = nominalTermStructure_->referenceDate();
         zciis_.reset(new ZeroCouponInflationSwap(Swap::Payer, nominal, start,
                                                  maturity_, calendar_, paymentConvention_,
                                                  dayCounter_, K, // fixed side & fixed rate
@@ -107,7 +104,7 @@ namespace QuantLib {
         // Because very simple instrument only takes
         // standard discounting swap engine.
         zciis_->setPricingEngine(
-            ext::shared_ptr<PricingEngine>(new DiscountingSwapEngine(nominalTS)));
+            ext::shared_ptr<PricingEngine>(new DiscountingSwapEngine(nominalTermStructure_)));
     }
 
 
@@ -158,7 +155,7 @@ namespace QuantLib {
 
 
     Real YearOnYearInflationSwapHelper::impliedQuote() const {
-        yyiis_->recalculate();
+        yyiis_->deepUpdate();
         return yyiis_->fairRate();
     }
 
@@ -202,13 +199,9 @@ namespace QuantLib {
 
         // The instrument takes a standard discounting swap engine.
         // The inflation-related work is done by the coupons.
-        QL_DEPRECATED_DISABLE_WARNING
-        Handle<YieldTermStructure> nominalTS =
-            !nominalTermStructure_.empty() ? nominalTermStructure_ : y->nominalTermStructure();
-        QL_DEPRECATED_ENABLE_WARNING
 
         yyiis_->setPricingEngine(
-            ext::shared_ptr<PricingEngine>(new DiscountingSwapEngine(nominalTS)));
+            ext::shared_ptr<PricingEngine>(new DiscountingSwapEngine(nominalTermStructure_)));
     }
 
 }
