@@ -28,19 +28,44 @@
 #include <ql/indexes/swapindex.hpp>
 #include <ql/instruments/makevanillaswap.hpp>
 #include <ql/instruments/simplifynotificationgraph.hpp>
+#include <ql/optional.hpp>
 #include <ql/pricingengines/swap/discountingswapengine.hpp>
 #include <ql/quote.hpp>
 #include <ql/termstructures/yield/ratehelpers.hpp>
 #include <ql/time/asx.hpp>
 #include <ql/time/calendars/jointcalendar.hpp>
-#include <ql/time/calendars/unitedstates.hpp>
 #include <ql/time/imm.hpp>
 #include <ql/utilities/null_deleter.hpp>
-#include <ql/optional.hpp>
 #include <utility>
 
 namespace QuantLib {
 
+    namespace {
+
+        void CheckDate(const Date& date, const Futures::Type type) {
+            switch (type) {
+              case Futures::IMM:
+                QL_REQUIRE(IMM::isIMMdate(date, false), date << " is not a valid IMM date");
+                break;
+              case Futures::ASX:
+                QL_REQUIRE(ASX::isASXdate(date, false), date << " is not a valid ASX date");
+                break;
+              case Futures::Custom:
+                break;
+              default:
+                QL_FAIL("unknown futures type (" << type << ')');
+            }
+        }
+
+        Time DetermineYearFraction(const Date& earliestDate,
+                                   const Date& maturityDate,
+                                   const DayCounter& dayCounter) {
+            return dayCounter.yearFraction(earliestDate, maturityDate,
+                                           earliestDate, maturityDate);
+        }
+
+    } // namespace
+
     FuturesRateHelper::FuturesRateHelper(const Handle<Quote>& price,
                                          const Date& iborStartDate,
                                          Natural lengthInMonths,
@@ -51,22 +76,12 @@ namespace QuantLib {
                                          Handle<Quote> convAdj,
                                          Futures::Type type)
     : RateHelper(price), convAdj_(std::move(convAdj)) {
-        switch (type) {
-          case Futures::IMM:
-            QL_REQUIRE(IMM::isIMMdate(iborStartDate, false),
-                       iborStartDate << " is not a valid IMM date");
-            break;
-          case Futures::ASX:
-            QL_REQUIRE(ASX::isASXdate(iborStartDate, false),
-                       iborStartDate << " is not a valid ASX date");
-            break;
-          default:
-            QL_FAIL("unknown futures type (" << Integer(type) << ")");
-        }
+        CheckDate(iborStartDate, type);
+
         earliestDate_ = iborStartDate;
-        maturityDate_ = calendar.advance(iborStartDate, lengthInMonths*Months,
-                                         convention, endOfMonth);
-        yearFraction_ = dayCounter.yearFraction(earliestDate_, maturityDate_);
+        maturityDate_ =
+            calendar.advance(iborStartDate, lengthInMonths * Months, convention, endOfMonth);
+        yearFraction_ = DetermineYearFraction(earliestDate_, maturityDate_, dayCounter);
         pillarDate_ = latestDate_ = latestRelevantDate_ = maturityDate_;
 
         registerWith(convAdj_);
@@ -81,27 +96,8 @@ namespace QuantLib {
                                          const DayCounter& dayCounter,
                                          Rate convAdj,
                                          Futures::Type type)
-    : RateHelper(price),
-      convAdj_(Handle<Quote>(ext::shared_ptr<Quote>(new SimpleQuote(convAdj))))
-    {
-        switch (type) {
-          case Futures::IMM:
-            QL_REQUIRE(IMM::isIMMdate(iborStartDate, false),
-                iborStartDate << " is not a valid IMM date");
-            break;
-          case Futures::ASX:
-            QL_REQUIRE(ASX::isASXdate(iborStartDate, false),
-                iborStartDate << " is not a valid ASX date");
-            break;
-          default:
-            QL_FAIL("unknown futures type (" << Integer(type) << ")");
-        }
-        earliestDate_ = iborStartDate;
-        maturityDate_ = calendar.advance(iborStartDate, lengthInMonths*Months,
-            convention, endOfMonth);
-        yearFraction_ = dayCounter.yearFraction(earliestDate_, maturityDate_);
-        pillarDate_ = latestDate_ = latestRelevantDate_ = maturityDate_;
-    }
+    : FuturesRateHelper(makeQuoteHandle(price), iborStartDate, lengthInMonths, calendar,
+                        convention, endOfMonth, dayCounter, makeQuoteHandle(convAdj), type) {}
 
     FuturesRateHelper::FuturesRateHelper(const Handle<Quote>& price,
                                          const Date& iborStartDate,
@@ -110,46 +106,39 @@ namespace QuantLib {
                                          Handle<Quote> convAdj,
                                          Futures::Type type)
     : RateHelper(price), convAdj_(std::move(convAdj)) {
+        CheckDate(iborStartDate, type);
+
+        const auto determineMaturityDate =
+            [&iborStartDate, &iborEndDate](const auto nextDateCalculator) -> Date {
+                Date maturityDate;
+                if (iborEndDate == Date()) {
+                    // advance 3 months
+                    maturityDate = nextDateCalculator(iborStartDate);
+                    maturityDate = nextDateCalculator(maturityDate);
+                    maturityDate = nextDateCalculator(maturityDate);
+                } else {
+                    QL_REQUIRE(iborEndDate > iborStartDate,
+                               "end date (" << iborEndDate << ") must be greater than start date ("
+                                            << iborStartDate << ')');
+                    maturityDate = iborEndDate;
+                }
+                return maturityDate;
+            };
+
         switch (type) {
           case Futures::IMM:
-            QL_REQUIRE(IMM::isIMMdate(iborStartDate, false),
-                       iborStartDate << " is not a valid IMM date");
-            if (iborEndDate == Date()) {
-                // advance 3 months
-                maturityDate_ = IMM::nextDate(iborStartDate, false);
-                maturityDate_ = IMM::nextDate(maturityDate_, false);
-                maturityDate_ = IMM::nextDate(maturityDate_, false);
-            }
-            else {
-                QL_REQUIRE(iborEndDate>iborStartDate,
-                           "end date (" << iborEndDate <<
-                           ") must be greater than start date (" <<
-                           iborStartDate << ")");
-                maturityDate_ = iborEndDate;
-            }
+            maturityDate_ = determineMaturityDate(
+                [](const Date date) -> Date { return IMM::nextDate(date, false); });
             break;
           case Futures::ASX:
-            QL_REQUIRE(ASX::isASXdate(iborStartDate, false),
-                       iborStartDate << " is not a valid ASX date");
-            if (iborEndDate == Date()) {
-                // advance 3 months
-                maturityDate_ = ASX::nextDate(iborStartDate, false);
-                maturityDate_ = ASX::nextDate(maturityDate_, false);
-                maturityDate_ = ASX::nextDate(maturityDate_, false);
-            }
-            else {
-                QL_REQUIRE(iborEndDate>iborStartDate,
-                           "end date (" << iborEndDate <<
-                           ") must be greater than start date (" <<
-                          iborStartDate << ")");
-                maturityDate_ = iborEndDate;
-            }
+            maturityDate_ = determineMaturityDate(
+                [](const Date date) -> Date { return ASX::nextDate(date, false); });
             break;
           default:
-            QL_FAIL("unknown futures type (" << Integer(type) << ")");
+            QL_FAIL("unsupported futures type (" << type << ')');
         }
         earliestDate_ = iborStartDate;
-        yearFraction_ = dayCounter.yearFraction(earliestDate_, maturityDate_);
+        yearFraction_ = DetermineYearFraction(earliestDate_, maturityDate_, dayCounter);
         pillarDate_ = latestDate_ = latestRelevantDate_ = maturityDate_;
 
         registerWith(convAdj_);
@@ -161,76 +150,22 @@ namespace QuantLib {
                                          const DayCounter& dayCounter,
                                          Rate convAdj,
                                          Futures::Type type)
-    : RateHelper(price),
-      convAdj_(Handle<Quote>(ext::shared_ptr<Quote>(new SimpleQuote(convAdj))))
-    {
-        switch (type) {
-          case Futures::IMM:
-            QL_REQUIRE(IMM::isIMMdate(iborStartDate, false),
-                       iborStartDate << " is not a valid IMM date");
-            if (iborEndDate == Date()) {
-                // advance 3 months
-                maturityDate_ = IMM::nextDate(iborStartDate, false);
-                maturityDate_ = IMM::nextDate(maturityDate_, false);
-                maturityDate_ = IMM::nextDate(maturityDate_, false);
-            }
-            else {
-                QL_REQUIRE(iborEndDate>iborStartDate,
-                           "end date (" << iborEndDate <<
-                           ") must be greater than start date (" <<
-                           iborStartDate << ")");
-                maturityDate_ = iborEndDate;
-            }
-            break;
-          case Futures::ASX:
-            QL_REQUIRE(ASX::isASXdate(iborStartDate, false),
-                iborStartDate << " is not a valid ASX date");
-            if (iborEndDate == Date()) {
-                // advance 3 months
-                maturityDate_ = ASX::nextDate(iborStartDate, false);
-                maturityDate_ = ASX::nextDate(maturityDate_, false);
-                maturityDate_ = ASX::nextDate(maturityDate_, false);
-            }
-            else {
-                QL_REQUIRE(iborEndDate>iborStartDate,
-                           "end date (" << iborEndDate <<
-                           ") must be greater than start date (" <<
-                           iborStartDate << ")");
-                maturityDate_ = iborEndDate;
-            }
-            break;
-          default:
-            QL_FAIL("unknown futures type (" << Integer(type) << ")");
-        }
-        earliestDate_ = iborStartDate;
-        yearFraction_ = dayCounter.yearFraction(earliestDate_, maturityDate_);
-        pillarDate_ = latestDate_ = latestRelevantDate_ = maturityDate_;
-    }
+    : FuturesRateHelper(makeQuoteHandle(price), iborStartDate, iborEndDate, dayCounter,
+                        makeQuoteHandle(convAdj), type) {}
 
     FuturesRateHelper::FuturesRateHelper(const Handle<Quote>& price,
                                          const Date& iborStartDate,
-                                         const ext::shared_ptr<IborIndex>& i,
+                                         const ext::shared_ptr<IborIndex>& index,
                                          const Handle<Quote>& convAdj,
                                          Futures::Type type)
     : RateHelper(price), convAdj_(convAdj) {
-        switch (type) {
-          case Futures::IMM:
-            QL_REQUIRE(IMM::isIMMdate(iborStartDate, false),
-                       iborStartDate << " is not a valid IMM date");
-            break;
-          case Futures::ASX:
-            QL_REQUIRE(ASX::isASXdate(iborStartDate, false),
-                       iborStartDate << " is not a valid ASX date");
-            break;
-          default:
-            QL_FAIL("unknown futures type (" << Integer(type) << ")");
-        }
+        CheckDate(iborStartDate, type);
+
         earliestDate_ = iborStartDate;
-        const Calendar& cal = i->fixingCalendar();
-        maturityDate_ = cal.advance(iborStartDate, i->tenor(),
-                                    i->businessDayConvention());
-        yearFraction_ = i->dayCounter().yearFraction(earliestDate_,
-                                                     maturityDate_);
+        const Calendar& cal = index->fixingCalendar();
+        maturityDate_ =
+            cal.advance(iborStartDate, index->tenor(), index->businessDayConvention());
+        yearFraction_ = DetermineYearFraction(earliestDate_, maturityDate_, index->dayCounter());
         pillarDate_ = latestDate_ = latestRelevantDate_ = maturityDate_;
 
         registerWith(convAdj);
@@ -238,42 +173,19 @@ namespace QuantLib {
 
     FuturesRateHelper::FuturesRateHelper(Real price,
                                          const Date& iborStartDate,
-                                         const ext::shared_ptr<IborIndex>& i,
+                                         const ext::shared_ptr<IborIndex>& index,
                                          Rate convAdj,
                                          Futures::Type type)
-    : RateHelper(price),
-      convAdj_(Handle<Quote>(ext::shared_ptr<Quote>(new SimpleQuote(convAdj))))
-    {
-        switch (type) {
-          case Futures::IMM:
-            QL_REQUIRE(IMM::isIMMdate(iborStartDate, false),
-                iborStartDate << " is not a valid IMM date");
-            break;
-          case Futures::ASX:
-            QL_REQUIRE(ASX::isASXdate(iborStartDate, false),
-                iborStartDate << " is not a valid ASX date");
-            break;
-          default:
-            QL_FAIL("unknown futures type (" << Integer(type) << ")");
-        }
-        earliestDate_ = iborStartDate;
-        const Calendar& cal = i->fixingCalendar();
-        maturityDate_ = cal.advance(iborStartDate, i->tenor(),
-                                    i->businessDayConvention());
-        yearFraction_ = i->dayCounter().yearFraction(earliestDate_,
-                                                     maturityDate_);
-        pillarDate_ = latestDate_ = latestRelevantDate_ = maturityDate_;
-    }
+    : FuturesRateHelper(makeQuoteHandle(price), iborStartDate, index, makeQuoteHandle(convAdj), type) {}
 
     Real FuturesRateHelper::impliedQuote() const {
         QL_REQUIRE(termStructure_ != nullptr, "term structure not set");
         Rate forwardRate = (termStructure_->discount(earliestDate_) /
             termStructure_->discount(maturityDate_) - 1.0) / yearFraction_;
-        Rate convAdj = convAdj_.empty() ? 0.0 : convAdj_->value();
         // Convexity, as FRA/futures adjustment, has been used in the
         // past to take into account futures margining vs FRA.
         // Therefore, there's no requirement for it to be non-negative.
-        Rate futureRate = forwardRate + convAdj;
+        Rate futureRate = forwardRate + convexityAdjustment();
         return 100.0 * (1.0 - futureRate);
     }
 
@@ -311,13 +223,8 @@ namespace QuantLib {
                                          BusinessDayConvention convention,
                                          bool endOfMonth,
                                          const DayCounter& dayCounter)
-    : RelativeDateRateHelper(rate) {
-        iborIndex_ = ext::make_shared<IborIndex>("no-fix", // never take fixing into account
-                      tenor, fixingDays,
-                      Currency(), calendar, convention,
-                      endOfMonth, dayCounter, termStructureHandle_);
-        DepositRateHelper::initializeDates();
-    }
+    : DepositRateHelper(makeQuoteHandle(rate), tenor, fixingDays, calendar, convention,
+                        endOfMonth, dayCounter) {}
 
     DepositRateHelper::DepositRateHelper(const Handle<Quote>& rate,
                                          const ext::shared_ptr<IborIndex>& i)
@@ -328,10 +235,7 @@ namespace QuantLib {
 
     DepositRateHelper::DepositRateHelper(Rate rate,
                                          const ext::shared_ptr<IborIndex>& i)
-    : RelativeDateRateHelper(rate) {
-        iborIndex_ = i->clone(termStructureHandle_);
-        DepositRateHelper::initializeDates();
-    }
+    : DepositRateHelper(makeQuoteHandle(rate), i) {}
 
     Real DepositRateHelper::impliedQuote() const {
         QL_REQUIRE(termStructure_ != nullptr, "term structure not set");
@@ -382,21 +286,12 @@ namespace QuantLib {
                                  Pillar::Choice pillarChoice,
                                  Date customPillarDate,
                                  bool useIndexedCoupon)
-    : RelativeDateRateHelper(rate), periodToStart_(monthsToStart*Months),
-      pillarChoice_(pillarChoice), useIndexedCoupon_(useIndexedCoupon) {
+    : FraRateHelper(rate, monthsToStart*Months, monthsToEnd-monthsToStart, fixingDays, calendar,
+        convention, endOfMonth, dayCounter, pillarChoice, customPillarDate, useIndexedCoupon) {
         QL_REQUIRE(monthsToEnd>monthsToStart,
                    "monthsToEnd (" << monthsToEnd <<
                    ") must be grater than monthsToStart (" << monthsToStart <<
                    ")");
-        // no way to take fixing into account,
-        // even if we would like to for FRA over today
-        iborIndex_ = ext::make_shared<IborIndex>("no-fix", // correct family name would be needed
-                      (monthsToEnd-monthsToStart)*Months,
-                      fixingDays,
-                      Currency(), calendar, convention,
-                      endOfMonth, dayCounter, termStructureHandle_);
-        pillarDate_ = customPillarDate;
-        FraRateHelper::initializeDates();
     }
 
     FraRateHelper::FraRateHelper(Rate rate,
@@ -410,22 +305,8 @@ namespace QuantLib {
                                  Pillar::Choice pillarChoice,
                                  Date customPillarDate,
                                  bool useIndexedCoupon)
-    : RelativeDateRateHelper(rate), periodToStart_(monthsToStart*Months),
-      pillarChoice_(pillarChoice), useIndexedCoupon_(useIndexedCoupon) {
-        QL_REQUIRE(monthsToEnd>monthsToStart,
-                   "monthsToEnd (" << monthsToEnd <<
-                   ") must be grater than monthsToStart (" << monthsToStart <<
-                   ")");
-        // no way to take fixing into account,
-        // even if we would like to for FRA over today
-        iborIndex_ = ext::make_shared<IborIndex>("no-fix", // correct family name would be needed
-                      (monthsToEnd-monthsToStart)*Months,
-                      fixingDays,
-                      Currency(), calendar, convention,
-                      endOfMonth, dayCounter, termStructureHandle_);
-        pillarDate_ = customPillarDate;
-        FraRateHelper::initializeDates();
-    }
+    : FraRateHelper(makeQuoteHandle(rate), monthsToStart, monthsToEnd, fixingDays, calendar,
+                    convention, endOfMonth, dayCounter, pillarChoice, customPillarDate, useIndexedCoupon) {}
 
     FraRateHelper::FraRateHelper(const Handle<Quote>& rate,
                                  Natural monthsToStart,
@@ -433,7 +314,62 @@ namespace QuantLib {
                                  Pillar::Choice pillarChoice,
                                  Date customPillarDate,
                                  bool useIndexedCoupon)
-    : RelativeDateRateHelper(rate), periodToStart_(monthsToStart*Months),
+    : FraRateHelper(rate, monthsToStart*Months, i, pillarChoice, customPillarDate, useIndexedCoupon)
+    {}
+
+    FraRateHelper::FraRateHelper(Rate rate,
+                                 Natural monthsToStart,
+                                 const ext::shared_ptr<IborIndex>& i,
+                                 Pillar::Choice pillarChoice,
+                                 Date customPillarDate,
+                                 bool useIndexedCoupon)
+    : FraRateHelper(makeQuoteHandle(rate), monthsToStart, i, pillarChoice, customPillarDate, useIndexedCoupon) {}
+
+    FraRateHelper::FraRateHelper(const Handle<Quote>& rate,
+                                 Period periodToStart,
+                                 Natural lengthInMonths,
+                                 Natural fixingDays,
+                                 const Calendar& calendar,
+                                 BusinessDayConvention convention,
+                                 bool endOfMonth,
+                                 const DayCounter& dayCounter,
+                                 Pillar::Choice pillarChoice,
+                                 Date customPillarDate,
+                                 bool useIndexedCoupon)
+    : RelativeDateRateHelper(rate), periodToStart_(periodToStart),
+      pillarChoice_(pillarChoice), useIndexedCoupon_(useIndexedCoupon) {
+        // no way to take fixing into account,
+        // even if we would like to for FRA over today
+        iborIndex_ = ext::make_shared<IborIndex>("no-fix", // correct family name would be needed
+                      lengthInMonths*Months,
+                      fixingDays,
+                      Currency(), calendar, convention,
+                      endOfMonth, dayCounter, termStructureHandle_);
+        pillarDate_ = customPillarDate;
+        FraRateHelper::initializeDates();
+    }
+
+    FraRateHelper::FraRateHelper(Rate rate,
+                                 Period periodToStart,
+                                 Natural lengthInMonths,
+                                 Natural fixingDays,
+                                 const Calendar& calendar,
+                                 BusinessDayConvention convention,
+                                 bool endOfMonth,
+                                 const DayCounter& dayCounter,
+                                 Pillar::Choice pillarChoice,
+                                 Date customPillarDate,
+                                 bool useIndexedCoupon)
+    : FraRateHelper(makeQuoteHandle(rate), periodToStart, lengthInMonths, fixingDays, calendar,
+                    convention, endOfMonth, dayCounter, pillarChoice, customPillarDate, useIndexedCoupon) {}
+
+    FraRateHelper::FraRateHelper(const Handle<Quote>& rate,
+                                 Period periodToStart,
+                                 const ext::shared_ptr<IborIndex>& i,
+                                 Pillar::Choice pillarChoice,
+                                 Date customPillarDate,
+                                 bool useIndexedCoupon)
+    : RelativeDateRateHelper(rate), periodToStart_(periodToStart),
       pillarChoice_(pillarChoice), useIndexedCoupon_(useIndexedCoupon) {
         // take fixing into account
         iborIndex_ = i->clone(termStructureHandle_);
@@ -447,103 +383,12 @@ namespace QuantLib {
     }
 
     FraRateHelper::FraRateHelper(Rate rate,
-                                 Natural monthsToStart,
-                                 const ext::shared_ptr<IborIndex>& i,
-                                 Pillar::Choice pillarChoice,
-                                 Date customPillarDate,
-                                 bool useIndexedCoupon)
-    : RelativeDateRateHelper(rate), periodToStart_(monthsToStart*Months),
-      pillarChoice_(pillarChoice), useIndexedCoupon_(useIndexedCoupon) {
-        // take fixing into account
-        iborIndex_ = i->clone(termStructureHandle_);
-        // see above
-        iborIndex_->unregisterWith(termStructureHandle_);
-        registerWith(iborIndex_);
-        pillarDate_ = customPillarDate;
-        FraRateHelper::initializeDates();
-    }
-
-    FraRateHelper::FraRateHelper(const Handle<Quote>& rate,
-                                 Period periodToStart,
-                                 Natural lengthInMonths,
-                                 Natural fixingDays,
-                                 const Calendar& calendar,
-                                 BusinessDayConvention convention,
-                                 bool endOfMonth,
-                                 const DayCounter& dayCounter,
-                                 Pillar::Choice pillarChoice,
-                                 Date customPillarDate,
-                                 bool useIndexedCoupon)
-    : RelativeDateRateHelper(rate), periodToStart_(periodToStart),
-      pillarChoice_(pillarChoice), useIndexedCoupon_(useIndexedCoupon) {
-        // no way to take fixing into account,
-        // even if we would like to for FRA over today
-        iborIndex_ = ext::make_shared<IborIndex>("no-fix", // correct family name would be needed
-                      lengthInMonths*Months,
-                      fixingDays,
-                      Currency(), calendar, convention,
-                      endOfMonth, dayCounter, termStructureHandle_);
-        pillarDate_ = customPillarDate;
-        FraRateHelper::initializeDates();
-    }
-
-    FraRateHelper::FraRateHelper(Rate rate,
-                                 Period periodToStart,
-                                 Natural lengthInMonths,
-                                 Natural fixingDays,
-                                 const Calendar& calendar,
-                                 BusinessDayConvention convention,
-                                 bool endOfMonth,
-                                 const DayCounter& dayCounter,
-                                 Pillar::Choice pillarChoice,
-                                 Date customPillarDate,
-                                 bool useIndexedCoupon)
-    : RelativeDateRateHelper(rate), periodToStart_(periodToStart),
-      pillarChoice_(pillarChoice), useIndexedCoupon_(useIndexedCoupon) {
-        // no way to take fixing into account,
-        // even if we would like to for FRA over today
-        iborIndex_ = ext::make_shared<IborIndex>("no-fix", // correct family name would be needed
-                      lengthInMonths*Months,
-                      fixingDays,
-                      Currency(), calendar, convention,
-                      endOfMonth, dayCounter, termStructureHandle_);
-        pillarDate_ = customPillarDate;
-        FraRateHelper::initializeDates();
-    }
-
-    FraRateHelper::FraRateHelper(const Handle<Quote>& rate,
                                  Period periodToStart,
                                  const ext::shared_ptr<IborIndex>& i,
                                  Pillar::Choice pillarChoice,
                                  Date customPillarDate,
                                  bool useIndexedCoupon)
-    : RelativeDateRateHelper(rate), periodToStart_(periodToStart),
-      pillarChoice_(pillarChoice), useIndexedCoupon_(useIndexedCoupon) {
-        // take fixing into account
-        iborIndex_ = i->clone(termStructureHandle_);
-        // see above
-        iborIndex_->unregisterWith(termStructureHandle_);
-        registerWith(iborIndex_);
-        pillarDate_ = customPillarDate;
-        FraRateHelper::initializeDates();
-    }
-
-    FraRateHelper::FraRateHelper(Rate rate,
-                                 Period periodToStart,
-                                 const ext::shared_ptr<IborIndex>& i,
-                                 Pillar::Choice pillarChoice,
-                                 Date customPillarDate,
-                                 bool useIndexedCoupon)
-    : RelativeDateRateHelper(rate), periodToStart_(periodToStart),
-      pillarChoice_(pillarChoice), useIndexedCoupon_(useIndexedCoupon) {
-        // take fixing into account
-        iborIndex_ = i->clone(termStructureHandle_);
-        // see above
-        iborIndex_->unregisterWith(termStructureHandle_);
-        registerWith(iborIndex_);
-        pillarDate_ = customPillarDate;
-        FraRateHelper::initializeDates();
-    }
+    : FraRateHelper(makeQuoteHandle(rate), periodToStart, i, pillarChoice, customPillarDate, useIndexedCoupon) {}
 
     FraRateHelper::FraRateHelper(const Handle<Quote>& rate,
                                  Natural immOffsetStart,
@@ -570,16 +415,8 @@ namespace QuantLib {
                                  Pillar::Choice pillarChoice,
                                  Date customPillarDate,
                                  bool useIndexedCoupon)
-    : RelativeDateRateHelper(rate), immOffsetStart_(immOffsetStart), immOffsetEnd_(immOffsetEnd),
-      pillarChoice_(pillarChoice), useIndexedCoupon_(useIndexedCoupon) {
-        // take fixing into account
-        iborIndex_ = i->clone(termStructureHandle_);
-        // see above
-        iborIndex_->unregisterWith(termStructureHandle_);
-        registerWith(iborIndex_);
-        pillarDate_ = customPillarDate;
-        FraRateHelper::initializeDates();
-    }
+    : FraRateHelper(makeQuoteHandle(rate), immOffsetStart, immOffsetEnd, i, pillarChoice,
+                    customPillarDate, useIndexedCoupon) {}
 
     Real FraRateHelper::impliedQuote() const {
         QL_REQUIRE(termStructure_ != nullptr, "term structure not set");
@@ -689,26 +526,11 @@ namespace QuantLib {
                                    Date customPillarDate,
                                    bool endOfMonth,
                                    const ext::optional<bool>& useIndexedCoupons)
-    : RelativeDateRateHelper(rate), settlementDays_(Null<Natural>()), tenor_(swapIndex->tenor()),
-      pillarChoice_(pillarChoice), calendar_(swapIndex->fixingCalendar()),
-      fixedConvention_(swapIndex->fixedLegConvention()),
-      fixedFrequency_(swapIndex->fixedLegTenor().frequency()),
-      fixedDayCount_(swapIndex->dayCounter()), spread_(std::move(spread)), endOfMonth_(endOfMonth),
-      fwdStart_(fwdStart), discountHandle_(std::move(discount)), useIndexedCoupons_(useIndexedCoupons) {
-        // take fixing into account
-        iborIndex_ = swapIndex->iborIndex()->clone(termStructureHandle_);
-        // We want to be notified of changes of fixings, but we don't
-        // want notifications from termStructureHandle_ (they would
-        // interfere with bootstrapping.)
-        iborIndex_->unregisterWith(termStructureHandle_);
-
-        registerWith(iborIndex_);
-        registerWith(spread_);
-        registerWith(discountHandle_);
-
-        pillarDate_ = customPillarDate;
-        SwapRateHelper::initializeDates();
-    }
+    : SwapRateHelper(rate, swapIndex->tenor(), swapIndex->fixingCalendar(),
+        swapIndex->fixedLegTenor().frequency(), swapIndex->fixedLegConvention(),
+        swapIndex->dayCounter(), swapIndex->iborIndex(), std::move(spread), fwdStart,
+        std::move(discount), Null<Natural>(), pillarChoice, customPillarDate, endOfMonth,
+        useIndexedCoupons) {}
 
     SwapRateHelper::SwapRateHelper(const Handle<Quote>& rate,
                                    const Period& tenor,
@@ -756,27 +578,8 @@ namespace QuantLib {
                                    Date customPillarDate,
                                    bool endOfMonth,
                                    const ext::optional<bool>& useIndexedCoupons)
-    : RelativeDateRateHelper(rate), settlementDays_(Null<Natural>()), tenor_(swapIndex->tenor()),
-      pillarChoice_(pillarChoice), calendar_(swapIndex->fixingCalendar()),
-      fixedConvention_(swapIndex->fixedLegConvention()),
-      fixedFrequency_(swapIndex->fixedLegTenor().frequency()),
-      fixedDayCount_(swapIndex->dayCounter()), spread_(std::move(spread)), endOfMonth_(endOfMonth),
-      fwdStart_(fwdStart), discountHandle_(std::move(discount)),
-      useIndexedCoupons_(useIndexedCoupons) {
-        // take fixing into account
-        iborIndex_ = swapIndex->iborIndex()->clone(termStructureHandle_);
-        // We want to be notified of changes of fixings, but we don't
-        // want notifications from termStructureHandle_ (they would
-        // interfere with bootstrapping.)
-        iborIndex_->unregisterWith(termStructureHandle_);
-
-        registerWith(iborIndex_);
-        registerWith(spread_);
-        registerWith(discountHandle_);
-
-        pillarDate_ = customPillarDate;
-        SwapRateHelper::initializeDates();
-    }
+    : SwapRateHelper(makeQuoteHandle(rate), swapIndex, std::move(spread), fwdStart,
+                     std::move(discount), pillarChoice, customPillarDate, endOfMonth, useIndexedCoupons) {}
 
     SwapRateHelper::SwapRateHelper(Rate rate,
                                    const Period& tenor,
@@ -793,32 +596,14 @@ namespace QuantLib {
                                    Date customPillarDate,
                                    bool endOfMonth,
                                    const ext::optional<bool>& useIndexedCoupons)
-    : RelativeDateRateHelper(rate), settlementDays_(settlementDays), tenor_(tenor),
-      pillarChoice_(pillarChoice), calendar_(std::move(calendar)),
-      fixedConvention_(fixedConvention), fixedFrequency_(fixedFrequency),
-      fixedDayCount_(std::move(fixedDayCount)), spread_(std::move(spread)), endOfMonth_(endOfMonth),
-      fwdStart_(fwdStart), discountHandle_(std::move(discount)),
-      useIndexedCoupons_(useIndexedCoupons) {
-
-        // take fixing into account
-        iborIndex_ = iborIndex->clone(termStructureHandle_);
-        // We want to be notified of changes of fixings, but we don't
-        // want notifications from termStructureHandle_ (they would
-        // interfere with bootstrapping.)
-        iborIndex_->unregisterWith(termStructureHandle_);
-
-        registerWith(iborIndex_);
-        registerWith(spread_);
-        registerWith(discountHandle_);
-
-        pillarDate_ = customPillarDate;
-        SwapRateHelper::initializeDates();
-    }
+    : SwapRateHelper(makeQuoteHandle(rate), tenor, std::move(calendar), fixedFrequency, fixedConvention,
+                     std::move(fixedDayCount), iborIndex, std::move(spread), fwdStart, std::move(discount), settlementDays,
+                     pillarChoice, customPillarDate, endOfMonth, useIndexedCoupons) {}
 
     void SwapRateHelper::initializeDates() {
 
         // 1. do not pass the spread here, as it might be a Quote
-        //    i.e. it can dinamically change
+        //    i.e. it can dynamically change
         // 2. input discount curve Handle might be empty now but it could
         //    be assigned a curve later; use a RelinkableHandle here
         swap_ = MakeVanillaSwap(tenor_, iborIndex_, 0.0, fwdStart_)
