@@ -16,7 +16,7 @@
  FOR A PARTICULAR PURPOSE.  See the license for more details.
 */
 
-#include "equityindex.hpp"
+#include "toplevelfixture.hpp"
 #include "utilities.hpp"
 #include <ql/indexes/equityindex.hpp>
 #include <ql/time/calendars/target.hpp>
@@ -26,50 +26,50 @@
 using namespace QuantLib;
 using namespace boost::unit_test_framework;
 
-namespace equityindex_test {
+BOOST_FIXTURE_TEST_SUITE(QuantLibTests, TopLevelFixture)
 
-    struct CommonVars {
+BOOST_AUTO_TEST_SUITE(EquityIndexTests)
 
-        Date today;
-        Calendar calendar;
-        DayCounter dayCount;
+struct CommonVars {
 
-        ext::shared_ptr<EquityIndex> equityIndex;
-        RelinkableHandle<YieldTermStructure> interestHandle;
-        RelinkableHandle<YieldTermStructure> dividendHandle;
-        ext::shared_ptr<Quote> spot;
-        RelinkableHandle<Quote> spotHandle;
+    Date today;
+    Calendar calendar;
+    DayCounter dayCount;
 
-        // utilities
+    ext::shared_ptr<EquityIndex> equityIndex;
+    RelinkableHandle<YieldTermStructure> interestHandle;
+    RelinkableHandle<YieldTermStructure> dividendHandle;
+    ext::shared_ptr<Quote> spot;
+    RelinkableHandle<Quote> spotHandle;
 
-        CommonVars(bool addTodaysFixing = true) {
-            calendar = TARGET();
-            dayCount = Actual365Fixed();
+    // utilities
 
-            equityIndex = ext::make_shared<EquityIndex>("eqIndex", calendar, interestHandle,
-                                                        dividendHandle, spotHandle);
+    CommonVars(bool addTodaysFixing = true) {
+        calendar = TARGET();
+        dayCount = Actual365Fixed();
 
-            today = calendar.adjust(Date(27, January, 2023));
+        equityIndex = ext::make_shared<EquityIndex>("eqIndex", calendar, interestHandle,
+                                                    dividendHandle, spotHandle);
+
+        today = calendar.adjust(Date(27, January, 2023));
             
-            if (addTodaysFixing)
-                equityIndex->addFixing(today, 8690.0);
+        if (addTodaysFixing)
+            equityIndex->addFixing(today, 8690.0);
 
-            Settings::instance().evaluationDate() = today;
+        Settings::instance().evaluationDate() = today;
 
-            interestHandle.linkTo(flatRate(0.03, dayCount));
-            dividendHandle.linkTo(flatRate(0.01, dayCount));
+        interestHandle.linkTo(flatRate(0.03, dayCount));
+        dividendHandle.linkTo(flatRate(0.01, dayCount));
             
-            spot = ext::make_shared<SimpleQuote>(8700.0);
-            spotHandle.linkTo(spot);
-        }
-    };
-}
+        spot = ext::make_shared<SimpleQuote>(8700.0);
+        spotHandle.linkTo(spot);
+    }
+};
 
-void EquityIndexTest::testTodaysFixing() {
+
+BOOST_AUTO_TEST_CASE(testTodaysFixing) {
     BOOST_TEST_MESSAGE("Testing today's fixing...");
 
-    using namespace equityindex_test;
-
     CommonVars vars;
     const Real tolerance = 1.0e-8;
 
@@ -90,11 +90,9 @@ void EquityIndexTest::testTodaysFixing() {
                     << "    expected forecast:    " << spot << "\n");
 }
 
-void EquityIndexTest::testTodaysFixingWithSpotAsProxy() {
+BOOST_AUTO_TEST_CASE(testTodaysFixingWithSpotAsProxy) {
     BOOST_TEST_MESSAGE("Testing today's fixing with spot as proxy...");
 
-    using namespace equityindex_test;
-
     CommonVars vars(false);
     const Real tolerance = 1.0e-8;
 
@@ -107,11 +105,9 @@ void EquityIndexTest::testTodaysFixingWithSpotAsProxy() {
                     << "    expected fixing:    " << spot << "\n");
 }
 
-void EquityIndexTest::testFixingForecast() {
+BOOST_AUTO_TEST_CASE(testFixingForecast) {
     BOOST_TEST_MESSAGE("Testing fixing forecast...");
 
-    using namespace equityindex_test;
-
     CommonVars vars;
     const Real tolerance = 1.0e-8;
 
@@ -128,11 +124,9 @@ void EquityIndexTest::testFixingForecast() {
                     << "    expected forecast:    " << expectedForecast << "\n");
 }
 
-void EquityIndexTest::testFixingForecastWithoutDividend() {
+BOOST_AUTO_TEST_CASE(testFixingForecastWithoutDividend) {
     BOOST_TEST_MESSAGE("Testing fixing forecast without dividend...");
 
-    using namespace equityindex_test;
-
     CommonVars vars;
     const Real tolerance = 1.0e-8;
 
@@ -151,11 +145,9 @@ void EquityIndexTest::testFixingForecastWithoutDividend() {
                     << "    expected forecast:    " << expectedForecast << "\n");
 }
 
-void EquityIndexTest::testFixingForecastWithoutSpot() {
+BOOST_AUTO_TEST_CASE(testFixingForecastWithoutSpot) {
     BOOST_TEST_MESSAGE("Testing fixing forecast without spot handle...");
 
-    using namespace equityindex_test;
-
     CommonVars vars;
     const Real tolerance = 1.0e-8;
 
@@ -175,11 +167,9 @@ void EquityIndexTest::testFixingForecastWithoutSpot() {
                     << "    expected forecast:    " << expectedForecast << "\n");
 }
 
-void EquityIndexTest::testFixingForecastWithoutSpotAndHistoricalFixing() {
+BOOST_AUTO_TEST_CASE(testFixingForecastWithoutSpotAndHistoricalFixing) {
     BOOST_TEST_MESSAGE("Testing fixing forecast without spot handle and historical fixing...");
 
-    using namespace equityindex_test;
-
     CommonVars vars(false);
 
     Date forecastedDate(20, May, 2030);
@@ -193,11 +183,9 @@ void EquityIndexTest::testFixingForecastWithoutSpotAndHistoricalFixing() {
             "Cannot forecast equity index, missing both spot and historical index"));
 }
 
-void EquityIndexTest::testSpotChange() {
+BOOST_AUTO_TEST_CASE(testSpotChange) {
     BOOST_TEST_MESSAGE("Testing spot change...");
 
-    using namespace equityindex_test;
-
     CommonVars vars;
     const Real tolerance = 1.0e-8;
 
@@ -217,11 +205,9 @@ void EquityIndexTest::testSpotChange() {
                     << "    expected spot:    " << vars.spot->value() << "\n");
 }
 
-void EquityIndexTest::testErrorWhenInvalidFixingDate() {
+BOOST_AUTO_TEST_CASE(testErrorWhenInvalidFixingDate) {
     BOOST_TEST_MESSAGE("Testing error when invalid fixing date is used...");
 
-    using namespace equityindex_test;
-
     CommonVars vars;
 
     BOOST_CHECK_EXCEPTION(
@@ -229,11 +215,9 @@ void EquityIndexTest::testErrorWhenInvalidFixingDate() {
         ExpectedErrorMessage("Fixing date January 1st, 2023 is not valid"));
 }
 
-void EquityIndexTest::testErrorWhenFixingMissing() {
+BOOST_AUTO_TEST_CASE(testErrorWhenFixingMissing) {
     BOOST_TEST_MESSAGE("Testing error when required fixing is missing...");
 
-    using namespace equityindex_test;
-
     CommonVars vars;
 
     BOOST_CHECK_EXCEPTION(
@@ -241,11 +225,9 @@ void EquityIndexTest::testErrorWhenFixingMissing() {
         ExpectedErrorMessage("Missing eqIndex fixing for January 2nd, 2023"));
 }
 
-void EquityIndexTest::testErrorWhenInterestHandleMissing() {
+BOOST_AUTO_TEST_CASE(testErrorWhenInterestHandleMissing) {
     BOOST_TEST_MESSAGE("Testing error when interest handle is missing...");
 
-    using namespace equityindex_test;
-
     CommonVars vars;
 
     Date forecastedDate(20, May, 2030);
@@ -259,11 +241,9 @@ void EquityIndexTest::testErrorWhenInterestHandleMissing() {
                               "null interest rate term structure set to this instance of eqIndex"));
 }
 
-void EquityIndexTest::testFixingObservability() {
+BOOST_AUTO_TEST_CASE(testFixingObservability) {
     BOOST_TEST_MESSAGE("Testing observability of index fixings...");
 
-    using namespace equityindex_test;
-
     CommonVars vars;
 
     ext::shared_ptr<EquityIndex> i1 =
@@ -281,11 +261,9 @@ void EquityIndexTest::testFixingObservability() {
         BOOST_FAIL("Observer was not notified of added equity index fixing");
 }
 
-void EquityIndexTest::testNoErrorIfTodayIsNotBusinessDay() {
+BOOST_AUTO_TEST_CASE(testNoErrorIfTodayIsNotBusinessDay) {
     BOOST_TEST_MESSAGE("Testing that no error is thrown if today is not a business day...");
 
-    using namespace equityindex_test;
-
     CommonVars vars;
 
     Date today(28, January, 2023);
@@ -299,22 +277,6 @@ void EquityIndexTest::testNoErrorIfTodayIsNotBusinessDay() {
     BOOST_REQUIRE_NO_THROW(vars.equityIndex->fixing(forecastedDate));
 }
 
-test_suite* EquityIndexTest::suite() {
-    auto* suite = BOOST_TEST_SUITE("Equity index tests");
+BOOST_AUTO_TEST_SUITE_END()
 
-    suite->add(QUANTLIB_TEST_CASE(&EquityIndexTest::testTodaysFixing));
-    suite->add(QUANTLIB_TEST_CASE(&EquityIndexTest::testTodaysFixingWithSpotAsProxy));
-    suite->add(QUANTLIB_TEST_CASE(&EquityIndexTest::testFixingForecast));
-    suite->add(QUANTLIB_TEST_CASE(&EquityIndexTest::testFixingForecastWithoutDividend));
-    suite->add(QUANTLIB_TEST_CASE(&EquityIndexTest::testFixingForecastWithoutSpot));
-    suite->add(
-        QUANTLIB_TEST_CASE(&EquityIndexTest::testFixingForecastWithoutSpotAndHistoricalFixing));
-    suite->add(QUANTLIB_TEST_CASE(&EquityIndexTest::testSpotChange));
-    suite->add(QUANTLIB_TEST_CASE(&EquityIndexTest::testErrorWhenInvalidFixingDate));
-    suite->add(QUANTLIB_TEST_CASE(&EquityIndexTest::testErrorWhenFixingMissing));
-    suite->add(QUANTLIB_TEST_CASE(&EquityIndexTest::testErrorWhenInterestHandleMissing));
-    suite->add(QUANTLIB_TEST_CASE(&EquityIndexTest::testFixingObservability));
-    suite->add(QUANTLIB_TEST_CASE(&EquityIndexTest::testNoErrorIfTodayIsNotBusinessDay));
-
-    return suite;
-}
+BOOST_AUTO_TEST_SUITE_END()
